"
This class is the logical representation of an LDAP Directory Server to
 GemStone.  It contains the following instance variables:
   uri               - A Uniform Resource Identifier for LDAP conforming to
                       RFC 4516.
   bindDN            - A string which is the distinguished name (DN) used to
                       bind to the server to perform lookups, or nil if anonymous
		       binds are to be used (no bindDN required).
   bindPW            - a ByteArray, which is the encrypted password for bindDN,
                       or nil if anonymous binds are to be used.

"
Class {
	#name : 'LdapDirectoryServer',
	#superclass : 'Object',
	#instVars : [
		'uri',
		'bindDN',
		'bindPW'
	],
	#classVars : [
		'AllLdapDirectoryServers'
	],
	#gs_reservedoop : '156673',
	#category : nil
}

{ #category : 'Private - Initializing' }
LdapDirectoryServer class >> _smartInitialize [
"This method should only be called once during filein or image upgrade."
  (AllLdapDirectoryServers == nil)
    ifTrue:[ | newArray |
      newArray := Array new .
      newArray _unsafeSetOop: 156929  "OOP_AllLdapDirectoryServers".
      newArray objectSecurityPolicy: (Globals at:#DataCuratorObjectSecurityPolicy) .
      AllLdapDirectoryServers := newArray .
      ^ 'created'
  ] .
  ^ 'exists'

]

{ #category : 'Accessing' }
LdapDirectoryServer class >> allServers [
  ^ AllLdapDirectoryServers

]

{ #category : 'Instance Creation' }
LdapDirectoryServer class >> basicNewWithUri: uri bindDN: aBindDn password: password [
"Creates a new instance of the receiver, but does not add the new instance
 to the list of LDAP servers used to process login requests."

| result |
result := super new.
result uri: uri copy ;
       bindDN: aBindDn copy ;
       password: password .

^ result

]

{ #category : 'Searching' }
LdapDirectoryServer class >> findServerWithUri: aUriString [
"Searches for an instance of LdapDirectoryServer with the given URI.
 Returns nil if the directory server was not found."

^ self allServers detect:[:e| e uri = aUriString ] ifNone:[ nil ]

]

{ #category : 'Illegal Operations' }
LdapDirectoryServer class >> new [

"Disallowed.  To create a new LdapDirectoryServer, use newWithUri:... instead."

self shouldNotImplement: #new

]

{ #category : 'Illegal Operations' }
LdapDirectoryServer class >> new: anInt [

"Disallowed.  To create a new LdapDirectoryServer, use newWithUri:... instead."
self shouldNotImplement: #new:

]

{ #category : 'Instance Creation' }
LdapDirectoryServer class >> newWithUri: uri bindDN: aBindDn password: password [
"Creates a new instance of the receiver and adds the resulting object to
 the list of LdapDirectoryServer objects used to authorize logins."

| result |
result := self basicNewWithUri: uri
               bindDN: aBindDn
	       password: password .
self allServers add: result .
^ result

]

{ #category : 'Removing' }
LdapDirectoryServer class >> removeServerWithUri: aUriString [
"Removes an instance of LdapDirectoryServer with the given URI.
 Returns true if the directory server was removed, false if it
 could not be found."

| server |
server := self findServerWithUri: aUriString .
server isNil
  ifTrue:[ ^ false "not found" ] .

self allServers removeIdentical: server .
^ true

]

{ #category : 'Connection Testing' }
LdapDirectoryServer class >> testConnectionToServer: uri bindDN: aBindDn password: password [
"Attempts to perform a bind using aBindDn and password to the LDAP server specified by
 uri.  Returns true if the connection was successful, otherwise returns false."

 | result |
 "Create a temporary instance."
result := self basicNewWithUri: uri
               bindDN: aBindDn
	       password: password .
^ result testBind

]

{ #category : 'Private' }
LdapDirectoryServer >> _encryptPassword: aString [
"Encrypt aString into a ByteArray. Store the ByteArray in the bindPW inst var.
 Returns the receiver."

<primitive: 1008>
self _validateClass: LdapDirectoryServer .
aString _validateClass: String .
self _primitiveFailed: #_encryptPassword: args: { aString } .
self _uncontinuableError

]

{ #category : 'Private' }
LdapDirectoryServer >> _testBind [

<primitive: 1009>
self _validateClass: LdapDirectoryServer .
self _primitiveFailed: #_testBind: args: { } .
self _uncontinuableError

]

{ #category : 'Accessing' }
LdapDirectoryServer >> bindDN [
  ^ bindDN

]

{ #category : 'Updating' }
LdapDirectoryServer >> bindDN: aString [
  bindDN := aString

]

{ #category : 'Accessing' }
LdapDirectoryServer >> bindPW [
  ^ bindPW

]

{ #category : 'Updating' }
LdapDirectoryServer >> password: aString [
^ aString == nil
    ifTrue:[ nil ]
   ifFalse:[ self _encryptPassword: aString ]

]

{ #category : 'Private' }
LdapDirectoryServer >> testBind [
"Attempt to bind to the specified LDAP server. Returns true if successful, false if
 the password is invalid.  Raises an error if an LDAP error occurs."

| result |
result := self _testBind .
result class == String
  ifTrue:[ IOError signal: result ].
^ result

]

{ #category : 'Accessing' }
LdapDirectoryServer >> uri [
  ^ uri

]

{ #category : 'Updating' }
LdapDirectoryServer >> uri: aString [
  uri := aString

]
